"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
Object.defineProperty(exports, "__esModule", { value: true });
// Imports.
const Command_1 = require("../talents/commander/src/Command/Command");
const CommandClientHandler_1 = require("../talents/commander/src/Command/CommandClientHandler");
const Instruction_1 = require("../talents/commander/src/Instruction/Instruction");
const CommandCatalogue_1 = require("../talents/commander/src/Service/CommandCatalogue");
const CommandComposer_1 = require("../talents/commander/src/Service/CommandComposer");
const CommandDirectoryLoader_1 = require("../talents/commander/src/Service/CommandDirectoryLoader");
const CommandManager_1 = require("../talents/commander/src/Service/CommandManager");
const CommandPluginSeeker_1 = require("../talents/commander/src/Service/CommandPluginSeeker");
const Listener_1 = require("../talents/listeners/src/Listener/Listener");
const ListenerCatalogue_1 = require("../talents/listeners/src/Service/ListenerCatalogue");
const ListenerFileLoader_1 = require("../talents/listeners/src/Service/ListenerFileLoader");
const ListenerManager_1 = require("../talents/listeners/src/Service/ListenerManager");
const ListenerPluginSeeker_1 = require("../talents/listeners/src/Service/ListenerPluginSeeker");
const ListenerResonator_1 = require("../talents/listeners/src/Service/ListenerResonator");
const Bot_1 = require("./Lavenza/Bot/Bot");
const BotCatalogue_1 = require("./Lavenza/Bot/BotCatalogue");
const BotDirectoryLoader_1 = require("./Lavenza/Bot/BotDirectoryLoader");
const BotManager_1 = require("./Lavenza/Bot/BotManager");
const Client_1 = require("./Lavenza/Client/Client");
const ClientType_1 = require("./Lavenza/Client/ClientType");
const DiscordPrompt_1 = require("./Lavenza/Client/Discord/DiscordPrompt");
const DiscordResonance_1 = require("./Lavenza/Client/Discord/DiscordResonance");
const TwitchPrompt_1 = require("./Lavenza/Client/Twitch/TwitchPrompt");
const TwitchResonance_1 = require("./Lavenza/Client/Twitch/TwitchResonance");
const Akechi_1 = require("./Lavenza/Confidant/Akechi");
const Igor_1 = require("./Lavenza/Confidant/Igor");
const Kawakami_1 = require("./Lavenza/Confidant/Kawakami");
const Morgana_1 = require("./Lavenza/Confidant/Morgana");
const Sojiro_1 = require("./Lavenza/Confidant/Sojiro");
const Yoshida_1 = require("./Lavenza/Confidant/Yoshida");
const Core_1 = require("./Lavenza/Core/Core");
const CoreStatus_1 = require("./Lavenza/Core/CoreStatus");
const Eminence_1 = require("./Lavenza/Eminence/Eminence");
const FunctionalResonator_1 = require("./Lavenza/Functional/FunctionalResonator");
const Composer_1 = require("./Lavenza/Gestalt/Composer/Composer");
const Gestalt_1 = require("./Lavenza/Gestalt/Gestalt");
const GestaltComposer_1 = require("./Lavenza/Gestalt/GestaltComposer");
const GestaltEventSubscriber_1 = require("./Lavenza/Gestalt/GestaltEventSubscriber");
const StorageService_1 = require("./Lavenza/Gestalt/StorageService/StorageService");
const PromptException_1 = require("./Lavenza/Prompt/Exception/PromptException");
const PromptExceptionType_1 = require("./Lavenza/Prompt/Exception/PromptExceptionType");
const Prompt_1 = require("./Lavenza/Prompt/Prompt");
const PromptResonator_1 = require("./Lavenza/Prompt/PromptResonator");
const Resonance_1 = require("./Lavenza/Resonance/Resonance");
const ResonanceEventSubscriber_1 = require("./Lavenza/Resonance/ResonanceEventSubscriber");
const Resonator_1 = require("./Lavenza/Resonance/Resonator/Resonator");
const Catalogue_1 = require("./Lavenza/Service/Catalogue/Catalogue");
const EventSubscriber_1 = require("./Lavenza/Service/EventSubscriber/EventSubscriber");
const EventSubscriberManager_1 = require("./Lavenza/Service/EventSubscriber/EventSubscriberManager");
const SubscriptionRecordCatalogue_1 = require("./Lavenza/Service/EventSubscriber/SubscriptionRecordCatalogue");
const DirectoryLoader_1 = require("./Lavenza/Service/Loader/DirectoryLoader");
const FileLoader_1 = require("./Lavenza/Service/Loader/FileLoader");
const Loader_1 = require("./Lavenza/Service/Loader/Loader");
const PluginSeeker_1 = require("./Lavenza/Service/PluginSeeker/PluginSeeker");
const PluginSeekerManager_1 = require("./Lavenza/Service/PluginSeeker/PluginSeekerManager");
const RuntimeProcessId_1 = require("./Lavenza/Service/RuntimeProcessId");
const Service_1 = require("./Lavenza/Service/Service");
const ServiceContainer_1 = require("./Lavenza/Service/ServiceContainer");
const Talent_1 = require("./Lavenza/Talent/Talent");
const TalentCatalogue_1 = require("./Lavenza/Talent/TalentCatalogue");
const TalentDirectoryLoader_1 = require("./Lavenza/Talent/TalentDirectoryLoader");
const TalentManager_1 = require("./Lavenza/Talent/TalentManager");
// Define the module's exports.
module.exports = {
    // Lavenza's core.
    // This class is the main handler of the application.
    // There is a clear defined order as to how things are ran in the application. The Core properly outlines this order.
    Core: Core_1.Core,
    ServiceContainer: ServiceContainer_1.ServiceContainer,
    // Confidants.
    // Re-usable functionality is managed in what I'm calling Confidants for this project. Shoutouts to Persona 5!
    // Each confidant has a specific use. See each of their files for more deets.
    // Adding them here for ease of access in other applications.
    Akechi: Akechi_1.Akechi,
    Igor: Igor_1.Igor,
    Kawakami: Kawakami_1.Kawakami,
    Morgana: Morgana_1.Morgana,
    Sojiro: Sojiro_1.Sojiro,
    Yoshida: Yoshida_1.Yoshida,
    // Services.
    BotCatalogue: BotCatalogue_1.BotCatalogue,
    BotDirectoryLoader: BotDirectoryLoader_1.BotDirectoryLoader,
    BotManager: BotManager_1.BotManager,
    CommandCatalogue: CommandCatalogue_1.CommandCatalogue,
    CommandComposer: CommandComposer_1.CommandComposer,
    CommandDirectoryLoader: CommandDirectoryLoader_1.CommandDirectoryLoader,
    CommandManager: CommandManager_1.CommandManager,
    CommandPluginSeeker: CommandPluginSeeker_1.CommandPluginSeeker,
    DirectoryLoader: DirectoryLoader_1.DirectoryLoader,
    EventSubscriberManager: EventSubscriberManager_1.EventSubscriberManager,
    FileLoader: FileLoader_1.FileLoader,
    FunctionalResonator: FunctionalResonator_1.FunctionalResonator,
    Gestalt: Gestalt_1.Gestalt,
    GestaltComposer: GestaltComposer_1.GestaltComposer,
    GestaltEventSubscriber: GestaltEventSubscriber_1.GestaltEventSubscriber,
    ListenerCatalogue: ListenerCatalogue_1.ListenerCatalogue,
    ListenerFileLoader: ListenerFileLoader_1.ListenerFileLoader,
    ListenerManager: ListenerManager_1.ListenerManager,
    ListenerPluginSeeker: ListenerPluginSeeker_1.ListenerPluginSeeker,
    ListenerResonator: ListenerResonator_1.ListenerResonator,
    PluginSeekerManager: PluginSeekerManager_1.PluginSeekerManager,
    PromptResonator: PromptResonator_1.PromptResonator,
    ResonanceEventSubscriber: ResonanceEventSubscriber_1.ResonanceEventSubscriber,
    SubscriptionRecordCatalogue: SubscriptionRecordCatalogue_1.SubscriptionRecordCatalogue,
    TalentCatalogue: TalentCatalogue_1.TalentCatalogue,
    TalentDirectoryLoader: TalentDirectoryLoader_1.TalentDirectoryLoader,
    TalentManager: TalentManager_1.TalentManager,
    // Classes & Models.
    // These are classes that are extended or used across the application. We import them here once.
    // They are linked in the global variable for easy access to outside applications.
    Bot: Bot_1.Bot,
    Catalogue: Catalogue_1.Catalogue,
    Client: Client_1.Client,
    Command: Command_1.Command,
    CommandClientHandler: CommandClientHandler_1.CommandClientHandler,
    Composer: Composer_1.Composer,
    DiscordPrompt: DiscordPrompt_1.DiscordPrompt,
    DiscordResonance: DiscordResonance_1.DiscordResonance,
    EventSubscriber: EventSubscriber_1.EventSubscriber,
    Instruction: Instruction_1.Instruction,
    Listener: Listener_1.Listener,
    Loader: Loader_1.Loader,
    PluginSeeker: PluginSeeker_1.PluginSeeker,
    Prompt: Prompt_1.Prompt,
    PromptException: PromptException_1.PromptException,
    Resonance: Resonance_1.Resonance,
    Resonator: Resonator_1.Resonator,
    Service: Service_1.Service,
    StorageService: StorageService_1.StorageService,
    Talent: Talent_1.Talent,
    TwitchPrompt: TwitchPrompt_1.TwitchPrompt,
    TwitchResonance: TwitchResonance_1.TwitchResonance,
    // Enums.
    ClientType: ClientType_1.ClientType,
    CoreStatus: CoreStatus_1.CoreStatus,
    Eminence: Eminence_1.Eminence,
    PromptExceptionType: PromptExceptionType_1.PromptExceptionType,
    RuntimeProcessId: RuntimeProcessId_1.RuntimeProcessId,
    // Utility functions.
    __: Yoshida_1.Yoshida.translate,
    initialize: Core_1.Core.initialize,
    personalize: Yoshida_1.Yoshida.personalize,
    service: Core_1.Core.service,
    summon: Core_1.Core.summon,
    translate: Yoshida_1.Yoshida.translate,
};
