"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BotCatalogue = void 0;
// Imports.
const Igor_1 = require("../Confidant/Igor");
const Sojiro_1 = require("../Confidant/Sojiro");
const Core_1 = require("../Core/Core");
const Catalogue_1 = require("../Service/Catalogue/Catalogue");
const ServiceContainer_1 = require("../Service/ServiceContainer");
const BotDirectoryLoader_1 = require("./BotDirectoryLoader");
/**
 * Provides a Catalogue for Bots.
 *
 * This class manages the storage and retrieval of bots.
 *
 * Bots are configured in the 'bots' folder at the root of the application.
 *
 */
class BotCatalogue extends Catalogue_1.Catalogue {
    /**
     * Genesis handler for the BotCatalogue Service.
     *
     * This runs in the genesis phase of the application.
     *
     * @see Core.summon();
     *
     * @inheritDoc
     */
    genesis() {
        return __awaiter(this, void 0, void 0, function* () {
            // Load all Bots from the specified installation directory.
            this.repository = yield ServiceContainer_1.ServiceContainer.get(BotDirectoryLoader_1.BotDirectoryLoader).load(Core_1.Core.paths.bots);
            // Check if we loaded duplicate bots and end if we do.
            if (Sojiro_1.Sojiro.arrayHasDuplicates(this.repository, "id")) {
                yield Igor_1.Igor.throw("Duplicate bots were loaded during the build phase. Please make sure you don't have duplicated bots!");
            }
        });
    }
    /**
     * Fetch all of the summoned bots from the catalogue.
     *
     * @returns
     *   An array of all bots that are currently summoned.
     */
    allSummoned() {
        return this.all().filter((bot) => bot.summoned === true);
    }
    /**
     * Fetch a Bot from the repository.
     *
     * @param id
     *   ID of the Bot to obtain.
     *
     * @returns
     *   Bot found in the repository. Returns undefined if no Talent is found.
     */
    getBot(id) {
        return this.find((bot) => bot.id === id);
    }
}
exports.BotCatalogue = BotCatalogue;
