"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BotDirectoryLoader = void 0;
// Modules.
const path = require("path");
const Akechi_1 = require("../Confidant/Akechi");
const Igor_1 = require("../Confidant/Igor");
const Morgana_1 = require("../Confidant/Morgana");
const Sojiro_1 = require("../Confidant/Sojiro");
const Core_1 = require("../Core/Core");
const DirectoryLoader_1 = require("../Service/Loader/DirectoryLoader");
const Bot_1 = require("./Bot");
/**
 * Provides a Directory Service Service for Talents.
 */
class BotDirectoryLoader extends DirectoryLoader_1.DirectoryLoader {
    /**
     * Load a talent from a given directory found in the loader.
     *
     * @param botDirectoryPath
     *   Path to the directory housing the bot.
     *
     * @return
     *   The Bot loaded from the directory.
     */
    process(botDirectoryPath) {
        return __awaiter(this, void 0, void 0, function* () {
            // Get the bot name. This is in fact the name of the directory.
            const id = path.basename(botDirectoryPath);
            // We want to ignore the 'example' bot in all cases.
            if (id === "example") {
                return;
            }
            // Get the config file for the bot.
            const configFilePath = `${botDirectoryPath}/config.yml`;
            const config = yield Akechi_1.Akechi.readYamlFile(configFilePath).catch(Igor_1.Igor.continue);
            // If the configuration is empty, stop here.
            // @TODO - Use https://www.npmjs.com/package/validate to validate configurations.
            if (Sojiro_1.Sojiro.isEmpty(config)) {
                yield Morgana_1.Morgana.warn("Configuration file could not be loaded for following bot: {{bot}}", { bot: id });
                return;
            }
            // If the 'active' flag of the config is set and is not 'true', we don't activate this bot.
            if (config.active !== undefined && config.active === false) {
                yield Morgana_1.Morgana.warn("The {{bot}} bot has been set to inactive. It will not be loaded.", { bot: id });
                return;
            }
            // Instantiate and set the bot to the collection.
            const bot = new Bot_1.Bot(id, config, botDirectoryPath);
            if (bot.id === Core_1.Core.settings.config.bots.master) {
                bot.isMaster = true;
            }
            // Print a success message.
            yield Morgana_1.Morgana.success("The {{bot}} bot has successfully been loaded!", { bot: id });
            // Return the bot.
            return bot;
        });
    }
}
exports.BotDirectoryLoader = BotDirectoryLoader;
