"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GestaltEventSubscriber = void 0;
const ClientType_1 = require("../Client/ClientType");
const EventSubscriber_1 = require("../Service/EventSubscriber/EventSubscriber");
const ServiceContainer_1 = require("../Service/ServiceContainer");
const Gestalt_1 = require("./Gestalt");
/**
 * Provides an event subscriber for the Gestalt talent.
 */
class GestaltEventSubscriber extends EventSubscriber_1.EventSubscriber {
    /**
     * Build tasks for the gestalt event subscriber.
     */
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            this.gestaltService = ServiceContainer_1.ServiceContainer.get(Gestalt_1.Gestalt);
        });
    }
    /**
     * Get the list of subscribed events for given clients.
     */
    getEventSubcriptions() {
        return {
            [ClientType_1.ClientType.Discord]: {
                guildCreate: { method: "discordOnGuildCreate", priority: 5000 },
            },
            [ClientType_1.ClientType.Twitch]: {},
        };
    }
    /**
     * Handle tasks when the "onGuildCreate" event is emitted through a Bot's discord client.
     *
     * @param client
     *   The client that emitted the event.
     * @param guild
     *   The data passed through the event. This varies depending on the event that was emitted.
     */
    // tslint:disable-next-line:completed-docs
    discordOnGuildCreate(client, { guild }) {
        return __awaiter(this, void 0, void 0, function* () {
            // We start by syncing the guild configurations.
            const guilds = this.gestaltService.sync({}, `/bots/${client.bot.id}/clients/${client.type}/guilds`);
            const baseGuildConfig = {
                commandPrefix: yield client.bot.config.commandPrefix,
                name: guild.name,
                userEminences: {},
            };
            if (!(guild.id in guilds)) {
                guilds[guild.id] = baseGuildConfig;
            }
            yield this.gestaltService.update(`/bots/${client.bot.id}/clients/${client.type}/guilds`, guilds);
        });
    }
}
exports.GestaltEventSubscriber = GestaltEventSubscriber;
