"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PromptResonator = void 0;
const Resonator_1 = require("../Resonance/Resonator/Resonator");
/**
 * Provides a Resonator for Prompts.
 *
 * Prompts are a core functionality in the framework and will implement their own resonator.
 */
class PromptResonator extends Resonator_1.Resonator {
    constructor() {
        super(...arguments);
        /**
         * The priority of the Resonator. This determines the order in which Resonators will resonate.
         */
        this.priority = 5000;
    }
    /**
     * Each resonator service must implement this function to determine what to do upon hearing a resonance.
     */
    resonate(resonance) {
        return __awaiter(this, void 0, void 0, function* () {
            // Fire all of the bot's prompts, if any.
            // @TODO - Prompts can have a catalogue of their own.
            yield Promise.all(resonance.bot.prompts.map((prompt) => __awaiter(this, void 0, void 0, function* () {
                // Fire the listen function.
                yield prompt.listen(resonance);
            })));
        });
    }
}
exports.PromptResonator = PromptResonator;
