"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DirectoryLoader = void 0;
// Imports.
const Akechi_1 = require("../../Confidant/Akechi");
const Sojiro_1 = require("../../Confidant/Sojiro");
const Loader_1 = require("./Loader");
/**
 * Provides an abstract class for Directory Loaders.
 *
 * Directory Loaders crawl a path and perform specific tasks on each directory found in a path.
 */
class DirectoryLoader extends Loader_1.Loader {
    /**
     * Crawl the provided root path and process directories found in it.
     *
     * @inheritDoc
     */
    load(root) {
        return __awaiter(this, void 0, void 0, function* () {
            // If this directory doesn't exist, we simply return.
            if (!(yield Akechi_1.Akechi.directoryExists(root))) {
                return [];
            }
            const directories = yield Akechi_1.Akechi.getDirectoriesFrom(root);
            // We'll throw an error for this function if the 'Commands' directory doesn't exist or is empty.
            if (Sojiro_1.Sojiro.isEmpty(directories)) {
                return [];
            }
            // Initialize a variable to store all items we find.
            const items = [];
            // We'll now act on each directory found.
            yield Promise.all(directories.map((directory) => __awaiter(this, void 0, void 0, function* () {
                const item = yield this.process(directory);
                if (item) {
                    items.push(item);
                }
            })));
            return items;
        });
    }
}
exports.DirectoryLoader = DirectoryLoader;
