"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileLoader = void 0;
// Imports.
const Akechi_1 = require("../../Confidant/Akechi");
const Sojiro_1 = require("../../Confidant/Sojiro");
const Loader_1 = require("./Loader");
/**
 * Provides an abstract class for File Loaders.
 *
 * File Loaders crawl a path and perform specific tasks on each File found in a path.
 */
class FileLoader extends Loader_1.Loader {
    /**
     * Crawl the provided root path and process files found in it.
     *
     * @inheritDoc
     */
    load(root) {
        return __awaiter(this, void 0, void 0, function* () {
            // If the provided directory doesn't exist, we simply return.
            if (!(yield Akechi_1.Akechi.directoryExists(root))) {
                return [];
            }
            // Get the list of files at the path.
            let files = yield Akechi_1.Akechi.getFilesFrom(root);
            // Filter the obtained list to exclude files that are not of the type specified.
            files = files.filter((listenerPath) => listenerPath.endsWith(this.fileType));
            // We'll throw an error for this function if the 'Listeners' directory doesn't exist or is empty.
            // This error should be caught and handled above.
            if (Sojiro_1.Sojiro.isEmpty(files)) {
                return [];
            }
            // Initialize a variable to store all items we find.
            const items = [];
            // We'll now act on each file found.
            yield Promise.all(files.map((filePath) => __awaiter(this, void 0, void 0, function* () {
                const item = yield this.process(filePath);
                if (item) {
                    items.push(item);
                }
            })));
            return items;
        });
    }
}
exports.FileLoader = FileLoader;
