"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PluginSeeker = void 0;
const BotCatalogue_1 = require("../../Bot/BotCatalogue");
const Sojiro_1 = require("../../Confidant/Sojiro");
const TalentCatalogue_1 = require("../../Talent/TalentCatalogue");
const Service_1 = require("../Service");
const ServiceContainer_1 = require("../ServiceContainer");
/**
 * Provides an abstract class for Plugin Seekers.
 *
 * Talents can blend functionality between each other and offer added functionality to Bots with the use of Plugins.
 * Plugin Seekers can be defined that will loop through all loaded talents/bots and attempt to find specific classes at
 * a defined path. These classes will be loaded and can be acted upon.
 */
class PluginSeeker extends Service_1.Service {
    constructor() {
        super(...arguments);
        /**
         * Service Tags.
         *
         * Plugin Seekers should only have this tag anyways.
         */
        this.tags = ["plugin_seeker"];
    }
    /**
     * Build handler for Plugin Seekers.
     *
     * @inheritDoc
     */
    seek() {
        return __awaiter(this, void 0, void 0, function* () {
            // Loop through the Talents Catalogue
            yield Promise.all(ServiceContainer_1.ServiceContainer.get(TalentCatalogue_1.TalentCatalogue).all().map((talent) => __awaiter(this, void 0, void 0, function* () {
                // Apply the defined loader to the specified path.
                const seekerPath = `${talent.directory}/${this.path}`;
                const loader = ServiceContainer_1.ServiceContainer.get(this.loader);
                const loadedPlugins = yield loader.load(seekerPath);
                if (!Sojiro_1.Sojiro.isEmpty(loadedPlugins)) {
                    yield this.plug(loadedPlugins, talent);
                }
            })));
            // Loop through the Bots Catalogue
            yield Promise.all(ServiceContainer_1.ServiceContainer.get(BotCatalogue_1.BotCatalogue).all().map((bot) => __awaiter(this, void 0, void 0, function* () {
                // Apply the defined loader to the specified path.
                const seekerPath = `${bot.directory}/${this.path}`;
                const loader = ServiceContainer_1.ServiceContainer.get(this.loader);
                const loadedPlugins = yield loader.load(seekerPath);
                if (!Sojiro_1.Sojiro.isEmpty(loadedPlugins)) {
                    yield this.plug(loadedPlugins, bot);
                }
            })));
        });
    }
}
exports.PluginSeeker = PluginSeeker;
