"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PluginSeekerManager = void 0;
const Service_1 = require("../Service");
const ServiceContainer_1 = require("../ServiceContainer");
/**
 * Provides a manager for plugin seekers.
 */
class PluginSeekerManager extends Service_1.Service {
    /**
     * Run all plugin seeker services.
     *
     * This will simply make sure that all plugins are loaded for all defined Plugin Seekers.
     */
    static seekPlugins() {
        return __awaiter(this, void 0, void 0, function* () {
            // Now we want to obtain all Plugin Seeker services and fire them.
            // We put AbstractObject here since the typing of the PluginSeeker doesn't matter in this context.
            for (const service of ServiceContainer_1.ServiceContainer.getServicesWithTag("plugin_seeker")) {
                // For each plugin seeker, we run the seek() function.
                // This should run all plugin seekers and load them all wherever needed.
                yield service.seek();
            }
        });
    }
    /**
     * Run Genesis process handler.
     *
     * @inheritDoc
     */
    genesis() {
        return __awaiter(this, void 0, void 0, function* () {
            yield PluginSeekerManager.seekPlugins();
        });
    }
}
exports.PluginSeekerManager = PluginSeekerManager;
