"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Service = void 0;
/**
 * Provides a base class for Services.
 *
 * Services are functionality extenders that can be accessed through the core of the application.
 *
 * They are prepared at runtime and usually come with bundled functionality that can be accessed elsewhere in the code.
 */
class Service {
    /**
     * Service constructor.
     *
     * @param id
     *   ID of the service. This is the key given to it in the definition file.
     * @param dependencies
     *   The dependencies of this service.
     * @param tags
     *   The tags of this service.
     * @param talent
     *   Talent this service is linked to, if any.
     */
    constructor(id, dependencies = [], tags = [], talent) {
        /**
         * Dependencies of this service.
         *
         * @inheritDoc
         */
        this.dependencies = [];
        /**
         * Tags of this service.
         *
         * @inheritDoc
         */
        this.tags = [];
        this.id = id;
        this.dependencies = dependencies;
        this.tags = tags;
        this.talent = talent;
    }
    /**
     * Default build tasks for services.
     *
     * @inheritDoc
     */
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            // Do nothing by default. Each service can implement its own build tasks.
        });
    }
}
exports.Service = Service;
