"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TalentCatalogue = void 0;
const Igor_1 = require("../Confidant/Igor");
const Sojiro_1 = require("../Confidant/Sojiro");
const Core_1 = require("../Core/Core");
const Catalogue_1 = require("../Service/Catalogue/Catalogue");
const TalentDirectoryLoader_1 = require("./TalentDirectoryLoader");
/**
 * Provides a Catalogue for Talents.
 *
 * This Catalogue will load necessary talents, and make them available in the bots.
 */
class TalentCatalogue extends Catalogue_1.Catalogue {
    /**
     * Genesis handler for the TalentCatalogue Service.
     *
     * This runs in the genesis phase of the application.
     *
     * @see Core.summon();
     *
     * @inheritDoc
     */
    genesis() {
        return __awaiter(this, void 0, void 0, function* () {
            // Load all Talents from the specified installation directories.
            // This loads all talents provided in the Core Framework, as well as any custom talents defined by the user.
            const talentDirectoryLoader = Core_1.Core.service(TalentDirectoryLoader_1.TalentDirectoryLoader);
            this.repository = [...yield talentDirectoryLoader.load(Core_1.Core.paths.talents.core), ...yield talentDirectoryLoader.load(Core_1.Core.paths.talents.custom)];
            // Check if we loaded duplicate talents and end if we do.
            if (Sojiro_1.Sojiro.arrayHasDuplicates(this.repository, "machineName")) {
                yield Igor_1.Igor.throw("Duplicate talents were loaded during the build phase. Please make sure you don't have duplicate talents!");
            }
        });
    }
    /**
     * Obtain the list of Talents assigned to a specific Bot.
     *
     * @param bot
     *   Bot to get talents for.
     */
    getTalentsForBot(bot) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.library(`bot::${bot.id}`);
        });
    }
    /**
     * Assign a specific talent to a bot's library.
     *
     * @param talent
     *   Talent to assign.
     * @param bot
     *   Bot to assign the Talent to.
     */
    assignTalentToBot(talent, bot) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.assign(talent, `bot::${bot.id}`);
        });
    }
    /**
     * Fetch a Talent from the repository.
     *
     * @param machineName
     *   Machine name of the Talent to obtain.
     *
     * @returns
     *   Talent found in the repository. Returns undefined if no Talent is found.
     */
    getTalent(machineName) {
        return this.find((talent) => talent.machineName === machineName);
    }
}
exports.TalentCatalogue = TalentCatalogue;
