"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TalentDirectoryLoader = void 0;
// Modules.
const path = require("path");
// Imports.
const Akechi_1 = require("../Confidant/Akechi");
const Igor_1 = require("../Confidant/Igor");
const Sojiro_1 = require("../Confidant/Sojiro");
const DirectoryLoader_1 = require("../Service/Loader/DirectoryLoader");
/**
 * Provides a DirectoryLoader Service Service for Talents.
 */
class TalentDirectoryLoader extends DirectoryLoader_1.DirectoryLoader {
    /**
     * Load a talent from a given directory found in the loader.
     *
     * @param talentDirectoryPath
     *   Path to the directory housing the talent.
     *
     * @return
     *   The talent loaded from the directory.
     */
    process(talentDirectoryPath) {
        return __awaiter(this, void 0, void 0, function* () {
            // Get the machine name of the talent from the directory path.
            const talentMachineName = path.basename(talentDirectoryPath);
            // Get the info file for the talent.
            const configFilePath = `${talentDirectoryPath}/${talentMachineName}.config.yml`;
            const config = yield Akechi_1.Akechi.readYamlFile(configFilePath)
                .catch(Igor_1.Igor.continue);
            // If the info is empty, we gotta stop here. They are mandatory.
            if (Sojiro_1.Sojiro.isEmpty(config)) {
                yield Igor_1.Igor.throw("Configuration file could not be located for the {{talent}} talent.", { talent: path.basename(talentDirectoryPath) });
            }
            // Set the directory to the info. It's useful information to have in the Talent itself!
            config.directory = talentDirectoryPath;
            // Require the class and instantiate the Talent.
            let talent = yield Promise.resolve().then(() => require(`${talentDirectoryPath}/${config.class}`));
            talent = new talent[path.basename(config.class, ".js")]();
            // If the talent could not be loaded somehow, we end here.
            if (!talent) {
                yield Igor_1.Igor.throw("An error occurred when requiring the {{talent}} talent's class. Verify the Talent's info file.", { talent: path.basename(talentDirectoryPath) });
            }
            // Await building of the talent.
            // Talents have build tasks too that must be done asynchronously. We'll run them here.
            yield talent.build(config);
            // Push the talent into the repository.
            return talent;
        });
    }
}
exports.TalentDirectoryLoader = TalentDirectoryLoader;
