"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TalentResonator = void 0;
const Resonator_1 = require("../Resonance/Resonator/Resonator");
const ServiceContainer_1 = require("../Service/ServiceContainer");
const TalentCatalogue_1 = require("./TalentCatalogue");
/**
 * Provides a Resonator for Talents.
 */
class TalentResonator extends Resonator_1.Resonator {
    constructor() {
        super(...arguments);
        /**
         * The priority of the Resonator. This determines the order in which Resonators will resonate.
         */
        this.priority = 4800;
    }
    /**
     * Each resonator service must implement this function to determine what to do upon hearing a resonance.
     */
    resonate(resonance) {
        return __awaiter(this, void 0, void 0, function* () {
            // Get all talents assigned to this bot.
            const talents = yield ServiceContainer_1.ServiceContainer.get(TalentCatalogue_1.TalentCatalogue).getTalentsForBot(resonance.bot);
            // We want to sort each talent by priority real fast.
            talents.sort((a, b) => b.priority - a.priority);
            // We'll loop through all Talents assigned to this bot.
            for (const talent of talents) {
                // Only resonate if the talent has this client enabled.
                if (talent.config.clients === "*" || talent.config.clients.includes(resonance.client.type.replace(/^\w/, (c) => c.toUpperCase()))) {
                    // Run the Resonate function for the talent.
                    // Not all talents will implement this, but they can if they want to.
                    yield talent.resonate(resonance);
                }
            }
        });
    }
}
exports.TalentResonator = TalentResonator;
