"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Commander = void 0;
const Talent_1 = require("../../lib/Lavenza/Talent/Talent");
const ClientUtilityFactory_1 = require("./src/Command/Client/ClientUtilityFactory");
const CommandCooldownManager_1 = require("./src/Command/CommandCooldownManager/CommandCooldownManager");
/**
 * Core Talent to allow management of commands in a Bot.
 */
class Commander extends Talent_1.Talent {
    /**
     * Resonator function for the Commander talent.
     *
     * This will attempt to search for a command in a message and do that command if necessary.
     *
     * @inheritDoc
     */
    resonate(resonance) {
        return __awaiter(this, void 0, void 0, function* () {
            // Use the CommandInterpreter to find out if there's a command in the resonance.
            // If there's a command, the interpreter will return an instruction that will be assigned to the Resonance we heard.
            const interpreter = yield ClientUtilityFactory_1.ClientUtilityFactory.buildCommandInterpreter(resonance);
            const instruction = yield interpreter.interpret(resonance);
            // If there is no instruction, we do nothing after all.
            if (!instruction) {
                return;
            }
            // If the help option is used, we fire the help function of the command and return.
            if (instruction.arguments._.includes("help") || "help" in instruction.arguments) {
                // We don't use await here, or this would probably halt the whole program while the command executes.
                instruction.command.help(instruction)
                    .then(() => {
                    // Do nothing.
                });
                return;
            }
            // Now that we know a command has been found, we need to pass it through the right Authorizer.
            // We use a factory to build an appropriate authorizer.
            const authorizer = yield ClientUtilityFactory_1.ClientUtilityFactory.buildCommandAuthorizer(instruction);
            const authorized = yield authorizer.authorize();
            if (!authorized) {
                return;
            }
            // If a command was found and authorized, execute it.
            // We don't use await here, or this would probably halt the whole program while the command executes.
            instruction.command.execute(instruction, resonance)
                .then(() => __awaiter(this, void 0, void 0, function* () {
                // Activate cooldown for this command.
                // @TODO - We MAY need to take this out of the above then() and set it to execute synchronously. We'll see.
                CommandCooldownManager_1.CommandCooldownManager.setCooldown(instruction)
                    .then(() => __awaiter(this, void 0, void 0, function* () {
                    // Do nothing for now!
                }));
            }));
        });
    }
}
exports.Commander = Commander;
