"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DiscordCommandHelpHandler = void 0;
const Yoshida_1 = require("../../../../../../lib/Lavenza/Confidant/Yoshida");
const ServiceContainer_1 = require("../../../../../../lib/Lavenza/Service/ServiceContainer");
const CommandComposer_1 = require("../../../Service/CommandComposer");
const CommandHelpHandler_1 = require("../../CommandHelpHandler/CommandHelpHandler");
/**
 * Provides an abstract class for Help Handlers.
 *
 * Help handlers are simple classes that determine what to do when the help argument is invoked for a command.
 */
class DiscordCommandHelpHandler extends CommandHelpHandler_1.CommandHelpHandler {
    /**
     * Method that handles what happens when the help argument is invoked.
     *
     * @param instruction
     *   Instruction to fire the help handler for.
     */
    help(instruction) {
        return __awaiter(this, void 0, void 0, function* () {
            // Variables & utility.
            const command = instruction.command;
            const bot = instruction.resonance.bot;
            const client = instruction.resonance.client;
            const resonance = instruction.resonance;
            // Get configuration.
            const config = yield ServiceContainer_1.ServiceContainer.get(CommandComposer_1.CommandComposer).getActiveCommandConfigForBot(command, bot);
            const parameterConfig = yield yield ServiceContainer_1.ServiceContainer.get(CommandComposer_1.CommandComposer).getActiveParameterConfigForBot(command, bot);
            // Start building the usage text by getting the command prefix.
            let usageText = `\`${instruction.prefix}${config.key}`;
            // If there is input defined for this command, we will add them to the help text.
            if (parameterConfig.input) {
                parameterConfig.input.requests.every((request) => {
                    usageText += ` {${request.replace(" ", "_")
                        .toLowerCase()}}\`\n`;
                });
            }
            else {
                usageText += "`\n";
            }
            // If there are aliases defined for this command, add all usage examples to the help text.
            if (config.aliases) {
                const original = usageText;
                config.aliases.every((alias) => {
                    usageText += original.replace(`${config.key}`, alias);
                    return true;
                });
            }
            // Set the usage section.
            const fields = [
                {
                    name: yield Yoshida_1.Yoshida.translate("Usage", resonance.locale),
                    text: usageText,
                },
            ];
            // If there are options defined for this command, we add a section for options.
            if (parameterConfig.options) {
                let optionsList = "";
                yield Promise.all(parameterConfig.options.map((option) => __awaiter(this, void 0, void 0, function* () {
                    const description = yield Yoshida_1.Yoshida.translate(option.description, resonance.locale);
                    const name = yield Yoshida_1.Yoshida.translate(option.name, resonance.locale);
                    optionsList += `**${name}** \`-${option.key} {${option.expects.replace(" ", "_")
                        .toLowerCase()}}\` - ${description}\n\n`;
                })));
                fields.push({
                    name: yield Yoshida_1.Yoshida.translate("Options", resonance.locale),
                    text: optionsList,
                });
            }
            // If there are flags defi-...You get the idea.
            if (parameterConfig.flags) {
                let flagsList = "";
                yield Promise.all(parameterConfig.flags.map((flag) => __awaiter(this, void 0, void 0, function* () {
                    const description = yield Yoshida_1.Yoshida.translate(flag.description, resonance.locale);
                    const name = yield Yoshida_1.Yoshida.translate(flag.name, resonance.locale);
                    flagsList += `**${name}** \`-${flag.key}\` - ${description}\n\n`;
                })));
                fields.push({
                    name: yield Yoshida_1.Yoshida.translate("Flags", resonance.locale),
                    text: flagsList,
                });
            }
            // Finally, send the embed.
            yield client.sendEmbed(resonance.message.channel, {
                description: yield Yoshida_1.Yoshida.translate(`${config.description}`, resonance.locale),
                fields,
                header: {
                    icon: client.connector.user.avatarURL,
                    text: yield Yoshida_1.Yoshida.translate("{{bot}} Guide", { bot: bot.config.name }, resonance.locale),
                },
                thumbnail: client.connector.user.avatarURL,
                title: yield Yoshida_1.Yoshida.translate(`${config.name}`, resonance.locale),
            });
        });
    }
}
exports.DiscordCommandHelpHandler = DiscordCommandHelpHandler;
