"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DiscordCommandInterpreter = void 0;
const GestaltComposer_1 = require("../../../../../../lib/Lavenza/Gestalt/GestaltComposer");
const ServiceContainer_1 = require("../../../../../../lib/Lavenza/Service/ServiceContainer");
const CommandInterpreter_1 = require("../../CommandInterpreter/CommandInterpreter");
/**
 * Provides a Command Interpreter for Commands in Discord.
 */
class DiscordCommandInterpreter extends CommandInterpreter_1.CommandInterpreter {
    /**
     * Get the command prefix for a specific client from a resonance.
     *
     * Each command interpreter for a specific client must implement its own way to find this, as configurations may
     * differ.
     *
     * @param resonance
     *   The Resonance we're taking a look at. We'll look at the information for the given bot to determine the prefix.
     *
     * @returns
     *   Returns the command prefix we need to check for.
     */
    getCommandPrefixForClient(resonance) {
        return __awaiter(this, void 0, void 0, function* () {
            // Get client specific configurations.
            const clientConfig = yield ServiceContainer_1.ServiceContainer.get(GestaltComposer_1.GestaltComposer).getActiveConfigForClient(resonance.client);
            if (resonance.message.guild && clientConfig.guilds[resonance.message.guild.id]) {
                return clientConfig.guilds[resonance.message.guild.id].commandPrefix || undefined;
            }
        });
    }
}
exports.DiscordCommandInterpreter = DiscordCommandInterpreter;
