"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Instruction = void 0;
/**
 * Provides a class for Instructions, entities that regroup information about a command that was interpreted from a
 * resonance.
 *
 * This class will contain relevant information surrounding the command called.
 *
 * Instructions are sent for approval before they carry out the command.
 */
class Instruction {
    /**
     * Constructor for Instructions.
     *
     * @param resonance
     *   The Resonance that this instruction was parsed from.
     * @param command
     *   The command that was loaded.
     * @param prefix
     *   The command prefix used to build this instruction.
     * @param config
     *   The command configurations.
     * @param args
     *   The arguments parsed from the command.
     * @param content
     *   The raw content of the instruction.
     */
    constructor(resonance, command, prefix, config, args, content) {
        this.resonance = resonance;
        this.config = config;
        this.command = command;
        this.arguments = args;
        this.content = content;
    }
}
exports.Instruction = Instruction;
