"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommandCatalogue = void 0;
// Imports.
const Bot_1 = require("../../../../lib/Lavenza/Bot/Bot");
const Catalogue_1 = require("../../../../lib/Lavenza/Service/Catalogue/Catalogue");
const Talent_1 = require("../../../../lib/Lavenza/Talent/Talent");
/**
 * Provides a static class with helper functions pertaining to commands.
 */
class CommandCatalogue extends Catalogue_1.Catalogue {
    /**
     * Store an array of listeners in a library designed for a given bot.
     *
     * @param commands
     *   Array of commands.
     * @param entity
     *   Entity to store the listeners for.
     */
    storeCommandsForEntity(commands, entity) {
        return __awaiter(this, void 0, void 0, function* () {
            // If a bot, we set the catalogue library id to the Bots's ID with a unique key.
            if (entity instanceof Bot_1.Bot) {
                yield this.store(commands, `bot::${entity.id}`);
            }
            // If a talent, we set the catalogue library id to the Talent's Machine Name with a unique key.
            if (entity instanceof Talent_1.Talent) {
                yield this.store(commands, `talent::${entity.machineName}`);
            }
        });
    }
    /**
     * Store an array of commands in a library designed for a given bot.
     *
     * @param entity
     *   Entity to get the commands for.
     */
    getCommandsForEntity(entity) {
        return __awaiter(this, void 0, void 0, function* () {
            // If a bot, we set the catalogue library id to the Bots's ID with a unique key.
            if (entity instanceof Bot_1.Bot) {
                return this.library(`bot::${entity.id}`);
            }
            // If a talent, we set the catalogue library id to the Talent's Machine Name with a unique key.
            if (entity instanceof Talent_1.Talent) {
                return this.library(`talent::${entity.machineName}`);
            }
        });
    }
}
exports.CommandCatalogue = CommandCatalogue;
