"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommandDirectoryLoader = void 0;
// Modules.
const path = require("path");
// Imports.
const Akechi_1 = require("../../../../lib/Lavenza/Confidant/Akechi");
const Igor_1 = require("../../../../lib/Lavenza/Confidant/Igor");
const Morgana_1 = require("../../../../lib/Lavenza/Confidant/Morgana");
const Sojiro_1 = require("../../../../lib/Lavenza/Confidant/Sojiro");
const DirectoryLoader_1 = require("../../../../lib/Lavenza/Service/Loader/DirectoryLoader");
/**
 * Provides a Directory Service Service for Commands.
 */
class CommandDirectoryLoader extends DirectoryLoader_1.DirectoryLoader {
    /**
     * Load a command from a given directory found in the loader.
     *
     * @param commandDirectoryPath
     *   Path to the directory housing the command.
     *
     * @return
     *   The loaded and instantiated Command.
     */
    process(commandDirectoryPath) {
        return __awaiter(this, void 0, void 0, function* () {
            // The name of the command will be the directory name.
            const name = path.basename(commandDirectoryPath);
            // The ID of the command will be its name in lowercase.
            const id = name.toLowerCase();
            // Get the config file for the command.
            // Each command should have a file with the format 'COMMAND_NAME.config.yml'.
            const configFilePath = `${commandDirectoryPath}/config.yml`;
            const config = yield Akechi_1.Akechi.readYamlFile(configFilePath)
                .catch(Igor_1.Igor.continue);
            // Stop if the configuration is empty or wasn't found.
            // We can't load the command without configurations.
            // @TODO - Use https://www.npmjs.com/package/validate to validate configurations.
            if (Sojiro_1.Sojiro.isEmpty(config)) {
                yield Morgana_1.Morgana.warn("Configuration file could not be loaded for the {{command}} command.", { command: name });
                return;
            }
            // Set directory to the configuration.
            // It's nice to have quick access to the command folder from within the command.
            config.directory = commandDirectoryPath;
            // If the configuration exists, we can process by loading the class of the Command.
            // If the class doesn't exist (this could be caused by the configuration being wrong), we stop.
            let command = yield Promise.resolve().then(() => require(`${commandDirectoryPath}/${config.class}`));
            if (Sojiro_1.Sojiro.isEmpty(command)) {
                yield Morgana_1.Morgana.warn("Class could not be loaded for the {{command}}", { command: name });
                return;
            }
            command = new command[path.basename(config.class, ".js")](id, config.key, commandDirectoryPath);
            // Now let's successfully register the command to the Talent.
            // Commands have build tasks too and are also singletons. We'll run them here.
            yield command.build(config);
            // Set the command to the list of commands we loaded.
            return command;
        });
    }
}
exports.CommandDirectoryLoader = CommandDirectoryLoader;
