"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommandManager = void 0;
// Imports.
const BotCatalogue_1 = require("../../../../lib/Lavenza/Bot/BotCatalogue");
const Service_1 = require("../../../../lib/Lavenza/Service/Service");
const ServiceContainer_1 = require("../../../../lib/Lavenza/Service/ServiceContainer");
const TalentCatalogue_1 = require("../../../../lib/Lavenza/Talent/TalentCatalogue");
const CommandCatalogue_1 = require("./CommandCatalogue");
/**
 * Provides a Manager Service for Commands.
 */
class CommandManager extends Service_1.Service {
    /**
     * Build handler for the CommandManager.
     *
     * @inheritDoc
     */
    synthesis() {
        return __awaiter(this, void 0, void 0, function* () {
            // Now we want to loop through all bots.
            // Our goal is to assign all listeners loaded into a bot's talents into the bot itself.
            for (const bot of ServiceContainer_1.ServiceContainer.get(BotCatalogue_1.BotCatalogue).all()) {
                // Loop in all Talents assigned to a specified bot.
                for (const talent of yield ServiceContainer_1.ServiceContainer.get(TalentCatalogue_1.TalentCatalogue).getTalentsForBot(bot)) {
                    // Assign the proper listeners.
                    const talentCommands = yield ServiceContainer_1.ServiceContainer.get(CommandCatalogue_1.CommandCatalogue).getCommandsForEntity(talent);
                    yield ServiceContainer_1.ServiceContainer.get(CommandCatalogue_1.CommandCatalogue).storeCommandsForEntity(talentCommands, bot);
                }
            }
        });
    }
}
exports.CommandManager = CommandManager;
