"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ListenerFileLoader = void 0;
// Modules.
const path = require("path");
// Imports.
const Morgana_1 = require("../../../../lib/Lavenza/Confidant/Morgana");
const FileLoader_1 = require("../../../../lib/Lavenza/Service/Loader/FileLoader");
/**
 * Provides an abstract class for special services that are used to discover bonds between Talents.
 */
class ListenerFileLoader extends FileLoader_1.FileLoader {
    constructor() {
        super(...arguments);
        /**
         * Store the type of file you wish to load.
         */
        this.fileType = ".js";
    }
    /**
     * Load a command from a given file found in the loader.
     *
     * @param listenerFilePath
     *   Path to the file defining the listener
     *
     * @return
     *   The instantiated Listener.
     */
    process(listenerFilePath) {
        return __awaiter(this, void 0, void 0, function* () {
            // We will simply require the file here.
            let listener = yield Promise.resolve().then(() => require(listenerFilePath));
            listener = new listener[path.basename(listenerFilePath, ".js")]();
            // If the require fails or the result is empty, we stop.
            if (!listener) {
                yield Morgana_1.Morgana.warn(`A Listener class could not be loaded: ${listenerFilePath}.`);
                return;
            }
            // Run listener build tasks.
            yield listener.build();
            // If everything goes smoothly, we return the listener.
            return listener;
        });
    }
}
exports.ListenerFileLoader = ListenerFileLoader;
