"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ListenerManager = void 0;
// Imports.
const BotCatalogue_1 = require("../../../../lib/Lavenza/Bot/BotCatalogue");
const Service_1 = require("../../../../lib/Lavenza/Service/Service");
const ServiceContainer_1 = require("../../../../lib/Lavenza/Service/ServiceContainer");
const TalentCatalogue_1 = require("../../../../lib/Lavenza/Talent/TalentCatalogue");
const ListenerCatalogue_1 = require("./ListenerCatalogue");
/**
 * Provides a Manager Service for Listeners.
 */
class ListenerManager extends Service_1.Service {
    /**
     * Build handler for the ListenerManager.
     *
     * @inheritDoc
     */
    synthesis() {
        return __awaiter(this, void 0, void 0, function* () {
            // Plugin are all loaded before synthesis in the genesis phase.
            // Now we want to loop through all bots.
            // Our goal is to assign all listeners loaded into a bot's talents into the bot itself.
            for (const bot of ServiceContainer_1.ServiceContainer.get(BotCatalogue_1.BotCatalogue).all()) {
                // Loop in all Talents assigned to a specified bot.
                for (const talent of yield ServiceContainer_1.ServiceContainer.get(TalentCatalogue_1.TalentCatalogue).getTalentsForBot(bot)) {
                    // Get the listeners assigned to the talent.
                    const talentListeners = yield ServiceContainer_1.ServiceContainer.get(ListenerCatalogue_1.ListenerCatalogue).getListenersForEntity(talent);
                    // Assign the Talent's listeners to the Bot.
                    yield ServiceContainer_1.ServiceContainer.get(ListenerCatalogue_1.ListenerCatalogue).storeListenersForEntity(talentListeners, bot);
                }
            }
        });
    }
}
exports.ListenerManager = ListenerManager;
