"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ListenerPluginSeeker = void 0;
const PluginSeeker_1 = require("../../../../lib/Lavenza/Service/PluginSeeker/PluginSeeker");
const ServiceContainer_1 = require("../../../../lib/Lavenza/Service/ServiceContainer");
const ListenerCatalogue_1 = require("./ListenerCatalogue");
/**
 * Provides a plugin seeker for Listeners.
 */
class ListenerPluginSeeker extends PluginSeeker_1.PluginSeeker {
    constructor() {
        super(...arguments);
        /**
         * The desired path relative to the talent's root directory where you'd like the service to search for Plugins.
         */
        this.path = "plugins/Listeners";
        /**
         * ID of a loader defined to act upon all items found in the defined path.
         *
         * This loader can be a FileLoader, DirectoryLoader or a custom defined loader that extends the Loader class.
         */
        this.loader = "listeners.loader";
    }
    /**
     * After plugins are loaded with the specified loader, you can customize what happens for each set of plugins.
     *
     * Each seeker must specify this function.
     *
     * @param plugins
     *   Loaded plugins, if any.
     * @param entity
     *   Entity these plugins were loaded for.
     */
    plug(plugins, entity) {
        return __awaiter(this, void 0, void 0, function* () {
            // Set loaded listeners up in the Listener Catalogue with the sections map.
            yield ServiceContainer_1.ServiceContainer.get(ListenerCatalogue_1.ListenerCatalogue).storeListenersForEntity(plugins, entity);
        });
    }
}
exports.ListenerPluginSeeker = ListenerPluginSeeker;
