"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ListenerResonator = void 0;
// Imports.
const Sojiro_1 = require("../../../../lib/Lavenza/Confidant/Sojiro");
const Resonator_1 = require("../../../../lib/Lavenza/Resonance/Resonator/Resonator");
const ServiceContainer_1 = require("../../../../lib/Lavenza/Service/ServiceContainer");
const ListenerCatalogue_1 = require("./ListenerCatalogue");
/**
 * Provides a resonator for Listeners.
 */
class ListenerResonator extends Resonator_1.Resonator {
    constructor() {
        super(...arguments);
        /**
         * The priority of the Resonator. This determines the order in which Resonators will resonate.
         */
        this.priority = 4000;
    }
    /**
     * Resonate function for the Listeners talent.
     *
     * Loop through all listeners in a bot and listen to the Resonance.
     *
     * @param resonance
     *   The resonance that was created from the client.
     */
    resonate(resonance) {
        return __awaiter(this, void 0, void 0, function* () {
            // Get all listeners for the bot.
            const listeners = yield ServiceContainer_1.ServiceContainer.get(ListenerCatalogue_1.ListenerCatalogue).getListenersForEntity(resonance.bot);
            // If there are no listeners, we can return.
            if (Sojiro_1.Sojiro.isEmpty(listeners)) {
                return;
            }
            // Run all listeners.
            yield Promise.all(listeners.map((listener) => listener.listen(resonance)));
        });
    }
}
exports.ListenerResonator = ListenerResonator;
